=== modified file 'drizzled/atomic/gcc_traits.h'
--- drizzled/atomic/gcc_traits.h	2009-07-17 20:59:45 +0000
+++ drizzled/atomic/gcc_traits.h	2010-03-31 19:04:12 +0000
@@ -32,36 +32,37 @@
 
   gcc_traits() {}
 
-  /* YES. I know these are semantically backwards...
-   * so... TODO: Ensure we're doing the "right" thing here
-   */
+  inline value_type add_and_fetch(volatile value_type *value, D addend )
+  {
+    return __sync_add_and_fetch(value, addend);
+  }
+
   inline value_type fetch_and_add(volatile value_type *value, D addend )
   {
-    return __sync_add_and_fetch(value, addend);
+    return __sync_fetch_and_add(value, addend);
   }
 
   inline value_type fetch_and_increment(volatile value_type *value)
   {
-    return __sync_add_and_fetch(value, 1);
+    return __sync_fetch_and_add(value, 1);
   }
 
   inline value_type fetch_and_decrement(volatile value_type *value)
   {
-    return __sync_sub_and_fetch(value, 1);
+    return __sync_fetch_and_sub(value, 1);
   }
 
   inline value_type fetch_and_store(volatile value_type *value,
                                     value_type new_value)
   {
-    /* TODO: Is this the right one? */
     return __sync_lock_test_and_set(value, new_value);
   }
 
-  inline value_type compare_and_swap(volatile value_type *value,
+  inline bool compare_and_swap(volatile value_type *value,
                                      value_type new_value,
                                      value_type comparand )
   {
-    return __sync_val_compare_and_swap(value, comparand, new_value);
+    return __sync_bool_compare_and_swap(value, comparand, new_value);
   }
 
   inline value_type fetch(const volatile value_type *value) const volatile
@@ -76,7 +77,7 @@
      * Look at how to rewrite the below to something that ICC feels is
      * OK and yet respects memory barriers.
      */
-    return __sync_add_and_fetch(const_cast<value_type *>(value), 0);
+    return __sync_fetch_and_add(const_cast<value_type *>(value), 0);
   }
 
   inline value_type store_with_release(volatile value_type *value,

=== modified file 'drizzled/atomic/pthread_traits.h'
--- drizzled/atomic/pthread_traits.h	2009-12-14 19:51:51 +0000
+++ drizzled/atomic/pthread_traits.h	2010-03-31 19:04:12 +0000
@@ -67,11 +67,20 @@
 
   pthread_traits() {}
 
+  inline value_type add_and_fetch(volatile value_type *value, D addend )
+  {
+    my_lock.lock();
+    *value += addend;
+    value_type ret= *value;
+    my_lock.unlock();
+    return ret;
+  }
+
   inline value_type fetch_and_add(volatile value_type *value, D addend )
   {
     my_lock.lock();
+    value_type ret= *value;
     *value += addend;
-    value_type ret= *value;
     my_lock.unlock();
     return ret;
   }
@@ -79,8 +88,8 @@
   inline value_type fetch_and_increment(volatile value_type *value)
   {
     my_lock.lock();
+    value_type ret= *value;
     *value++;
-    value_type ret= *value;
     my_lock.unlock();
     return ret;
   }
@@ -88,8 +97,8 @@
   inline value_type fetch_and_decrement(volatile value_type *value)
   {
     my_lock.lock();
+    value_type ret= *value;
     *value--;
-    value_type ret= *value;
     my_lock.unlock();
     return ret;
   }
@@ -98,27 +107,30 @@
                                     value_type new_value )
   {
     my_lock.lock();
+    value_type ret= *value;
     *value= new_value;
-    value_type ret= *value;
     my_lock.unlock();
     return ret;
   }
 
-  inline value_type compare_and_swap(volatile value_type *value,
+  inline bool compare_and_swap(volatile value_type *value,
                                      value_type new_value,
                                      value_type comparand )
   {
     my_lock.lock();
-    if (*value == comparand)
+    bool ret= (*value == comparand);
+    if (ret)
       *value= new_value;
-    value_type ret= *value;
     my_lock.unlock();
     return ret;
   }
 
   inline value_type fetch(const volatile value_type *value) const volatile
   {
-    return *value;
+    const_cast<mutex_wrapper>(my_lock).lock();
+    value_type ret= *value;
+    const_cast<mutex_wrapper>(my_lock).unlock();
+    return ret;
   }
 
   inline value_type store_with_release(volatile value_type *value,

=== modified file 'drizzled/atomics.h'
--- drizzled/atomics.h	2010-01-04 22:52:52 +0000
+++ drizzled/atomics.h	2010-03-31 19:04:12 +0000
@@ -58,6 +58,11 @@
 
   atomic_impl() : atomic_base<I>(), traits() {}
 
+  value_type add_and_fetch( D addend )
+  {
+    return traits.add_and_fetch(&this->my_value, addend);
+  }
+
   value_type fetch_and_add( D addend )
   {
     return traits.fetch_and_add(&this->my_value, addend);
@@ -78,7 +83,7 @@
     return traits.fetch_and_store(&this->my_value, value);
   }
 
-  value_type compare_and_swap( value_type value, value_type comparand )
+  bool compare_and_swap( value_type value, value_type comparand )
   {
     return traits.compare_and_swap(&this->my_value, value, comparand);
   }
@@ -102,7 +107,7 @@
 public:
   atomic_impl<I,D,T>& operator+=( D addend )
   {
-    fetch_and_add(addend)+addend;
+    increment(addend);
     return *this;
   }
 
@@ -113,15 +118,15 @@
     return operator+=(D(0)-addend);
   }
 
-  value_type increment() {
-    return fetch_and_add(1)+1;
-  }
-
-  value_type decrement() {
-    return fetch_and_add(D(-1))-1;
-  }
-
-
+  value_type increment()
+  {
+    return add_and_fetch(1);
+  }
+
+  value_type decrement()
+  {
+    return add_and_fetch(D(-1));
+  }
 };
 
 } /* namespace internal */

=== modified file 'drizzled/query_id.cc'
--- drizzled/query_id.cc	2010-03-31 18:05:47 +0000
+++ drizzled/query_id.cc	2010-03-31 19:04:12 +0000
@@ -27,7 +27,7 @@
 
 Query_id::Query_id()
 {
-  the_query_id= 0;
+  the_query_id= 1;
 }
 
 Query_id::~Query_id()

=== modified file 'drizzled/transaction_services.cc'
--- drizzled/transaction_services.cc	2010-03-26 22:24:40 +0000
+++ drizzled/transaction_services.cc	2010-03-31 19:04:12 +0000
@@ -913,7 +913,7 @@
 {
   message::TransactionContext *trx= in_transaction.mutable_transaction_context();
   trx->set_server_id(in_session->getServerId());
-  trx->set_transaction_id(in_session->getQueryId());
+  trx->set_transaction_id(getNextTransactionId());
   trx->set_start_timestamp(in_session->getCurrentTimestamp());
 }
 

=== modified file 'drizzled/transaction_services.h'
--- drizzled/transaction_services.h	2010-03-26 22:24:40 +0000
+++ drizzled/transaction_services.h	2010-03-31 19:04:12 +0000
@@ -25,6 +25,7 @@
 #ifndef DRIZZLED_TRANSACTION_SERVICES_H
 #define DRIZZLED_TRANSACTION_SERVICES_H
 
+#include "drizzled/atomics.h"
 #include "drizzled/message/transaction.pb.h"
 
 namespace drizzled
@@ -49,10 +50,17 @@
 {
 public:
   static const size_t DEFAULT_RECORD_SIZE= 100;
+  typedef uint64_t TransactionId;
   /**
    * Constructor
    */
-  TransactionServices() {}
+  TransactionServices()
+  {
+    /**
+     * @todo set transaction ID to the last one from an applier...
+     */
+    current_transaction_id= 0;
+  }
 
   /**
    * Singleton method
@@ -388,6 +396,23 @@
                                       plugin::MonitoredInTransaction *monitored,
                                       plugin::TransactionalStorageEngine *engine,
                                       plugin::XaResourceManager *resource_manager);
+  TransactionId getNextTransactionId()
+  {
+    return current_transaction_id.increment();
+  }
+  TransactionId getCurrentTransactionId()
+  {
+    return current_transaction_id;
+  }
+  /**
+   * DEBUG ONLY.  See plugin::TransactionLog::truncate()
+   */
+  void resetTransactionId()
+  {
+    current_transaction_id= 0;
+  }
+private:
+  atomic<TransactionId> current_transaction_id;
 };
 
 } /* namespace drizzled */

=== modified file 'plugin/transaction_log/transaction_log.cc'
--- plugin/transaction_log/transaction_log.cc	2010-03-31 18:05:47 +0000
+++ plugin/transaction_log/transaction_log.cc	2010-03-31 19:04:12 +0000
@@ -84,6 +84,7 @@
 #include <drizzled/errmsg_print.h>
 #include <drizzled/gettext.h>
 #include <drizzled/message/transaction.pb.h>
+#include <drizzled/transaction_services.h>
 #include <drizzled/algorithm/crc32.h>
 
 #include <google/protobuf/io/coded_stream.h>
@@ -192,12 +193,6 @@
    */
   off_t cur_offset= log_offset.fetch_and_add(static_cast<off_t>(data_length));
 
-  /*
-   * We adjust cur_offset back to the original log_offset before
-   * the increment above...
-   */
-  cur_offset-= static_cast<off_t>(data_length);
-
   /* 
    * Quick safety...if an error occurs above in another writer, the log 
    * file will be in a crashed state.
@@ -293,6 +288,7 @@
     result= ftruncate(log_file, log_offset);
   }
   while (result == -1 && errno == EINTR);
+  drizzled::TransactionServices::singleton().resetTransactionId();
 }
 
 bool TransactionLog::findLogFilenameContainingTransactionId(const ReplicationServices::GlobalTransactionId&,

